/*
 * PROJECT: napkit at http://matereal.sourceforge.jp/
 * ----------------------------------------------------------------------------
 *
 * This file is part of NyARToolkit Application Toolkit.
 * NyARToolkit Application Toolkit, or simply "napkit",
 * is a simple wrapper library for NyARToolkit.
 *
 * ----------------------------------------------------------------------------
 *
 * License version: GPL 3.0
 *
 * napkit is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * napkit is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with napkit. If not, see <http://www.gnu.org/licenses/>.
 */
package jp.digitalmuseum.napkit;

import java.awt.image.BufferedImage;

import jp.digitalmuseum.utils.ScreenRectangle;
import jp.digitalmuseum.utils.VariableLengthArray;
import jp.nyatla.nyartoolkit.NyARException;
import jp.nyatla.nyartoolkit.core.param.NyARParam;
import jp.nyatla.nyartoolkit.core.raster.rgb.INyARRgbRaster;
import jp.nyatla.nyartoolkit.core.rasterfilter.rgb2bin.NyARRasterFilter_ARToolkitThreshold;
import jp.nyatla.nyartoolkit.core.rasterreader.INyARBufferReader;
import jp.nyatla.nyartoolkit.core.squaredetect.INyARSquareDetector;
import jp.nyatla.nyartoolkit.core.squaredetect.NyARSquare;
import jp.nyatla.nyartoolkit.core.squaredetect.NyARSquareDetector_Rle;
import jp.nyatla.nyartoolkit.core.squaredetect.NyARSquareStack;
import jp.nyatla.nyartoolkit.core.types.NyARIntSize;


/**
 * Square detector class using NyARToolKit.
 *
 * @author Jun KATO
 */
public class NapSquareDetectorImpl implements NapSquareDetector {
	final private NyARParam param
			= getInitialCameraParameter();
	private final NyARSquareStack squareStack
			= new NyARSquareStack(MAX_SQUARE);
	private NyARRasterFilter_ARToolkitThreshold binarizationFilter;
	final private VariableLengthArray<ScreenRectangle> resultHolder
			= new VariableLengthArray<ScreenRectangle>();

	private INyARSquareDetector squareDetector;
	private NapIntRasterBinary binaryImage;
	private int threshold = THRESHOLD_DEFAULT;

	protected NapByteRasterBGR image;
	private long session;
	private long lastSession;

	/** Constructor. */
	public NapSquareDetectorImpl() {
		try {
			binarizationFilter = new NyARRasterFilter_ARToolkitThreshold(
					THRESHOLD_DEFAULT,
					INyARBufferReader.BUFFERFORMAT_BYTE1D_R8G8B8_24);
		} catch (NyARException e) {
			// Won't be reached here.
		}
		updateScreenSize();
	}

	/**
	 * Returns a default camera parameter object.
	 */
	private NyARParam getInitialCameraParameter() {
		final NyARParam p = new NyARParam();
		final NyARIntSize size = p.getScreenSize();
		size.w = 640;
		size.h = 480;
		p.setValue(new double[] {
			318.5,
			263.5,
			26.2,
			1.0127565206658486
		}, new double[] {
			700.9514702992245,
			0.0,
			316.5,
			0.0,
			0.0,
			726.0941816535367,
			241.5,
			0.0,
			0.0,
			0.0,
			1.0,
			0.0
		});
		return p;
	}

	/**
	 * Update screen size and initialize camera parameter related fields.
	 */
	private synchronized void updateScreenSize() {
		final NyARIntSize size = param.getScreenSize();
		try {
			squareDetector = new NyARSquareDetector_Rle(
					param.getDistortionFactor(), size);
		} catch (NyARException e) {
			// TODO When does this occur?
			e.printStackTrace();
		}
		image = new NapByteRasterBGR(size.w, size.h);
		binaryImage = new NapIntRasterBinary(size.w, size.h);
	}

	public void loadCameraParameter(String fileName) {

		// Back up the screen size.
		final NyARIntSize size = param.getScreenSize();
		final int width = size.w, height = size.h;

		// Load the parameter file.
		try {
			param.loadARParamFromFile(fileName);
		} catch (NyARException e) {
			throw new IllegalArgumentException(e);
		}

		// Restore the screen size.
		param.changeScreenSize(width, height);
		updateScreenSize();
	}

	public void loadCameraParameter(String fileName, int width, int height) {
		try {
			param.loadARParamFromFile(fileName);
		} catch (NyARException e) {
			throw new IllegalArgumentException(e);
		}
		param.changeScreenSize(width, height);
		updateScreenSize();
	}

	public int getWidth() {
		return param.getScreenSize().w;
	}

	public int getHeight() {
		return param.getScreenSize().h;
	}

	public void setSize(int width, int height) {
		param.changeScreenSize(width, height);
		updateScreenSize();
	}

	public int getThreshold() {
		return threshold;
	}

	public void setThreshold(int threshold) {
		this.threshold = threshold;
		binarizationFilter.setThreshold(threshold);
	}

	/**
	 * Binarize the image and detect squares.
	 *
	 * @param image
	 * @return
	 */
	synchronized NyARSquareStack detectRawSquare(INyARRgbRaster image) {
		session = (session + 1) % Integer.MAX_VALUE;
		try {
			binarizationFilter.doFilter(image, binaryImage);
			squareDetector.detectMarker(binaryImage, squareStack);
		} catch (NyARException e) {
			squareStack.clear();
		}
		return squareStack;
	}

	public VariableLengthArray<ScreenRectangle> detectSquare(byte[] imageData) {
		image.setData(imageData);
		detectRawSquare(image);
		updateResultHolder();
		return resultHolder;
	}

	private synchronized void updateResultHolder() {
		if (lastSession == session) {
			return;
		}
		final int length = squareStack.getLength();
		resultHolder.clear();
		resultHolder.realloc(length);
		for (int i = length - 1; i >= 0; i --) {
			final NyARSquare square = (NyARSquare) squareStack.getItem(i);
			resultHolder.push(NapUtils.convertToScreenRectangle(square));
		}
		lastSession = session;
	}

	public VariableLengthArray<ScreenRectangle> getLastSquareDetectionResult() {
		updateResultHolder();
		return new VariableLengthArray<ScreenRectangle>(resultHolder);
	}

	public synchronized BufferedImage getBinarizedImage() {
		if (binaryImage == null) {
			return null;
		}
		return binaryImage.getImage();
	}

	public long getSession() {
		return session;
	}
}
